#include <execxx.h>

#include <fstream>
#include <future>
#include <iostream>
#include <json.hpp>
#include <pqxx/pqxx>
#include <string>
#include <thread>
#include <vector>

#include "./utils.h"
#include "Log.h"
#include "interfaces/DatabaseConnection.h"
#include "interfaces/structs.h"
#include "server.h"

using json = nlohmann::json;

typedef std::vector<std::string> StringVec;
typedef std::pair<std::string, std::string> FieldPair;
typedef std::vector<std::pair<std::string, std::string> > FieldPairVector;

DatabaseConfiguration getConfiguration() {
  DatabaseCredentials credentials{};
  credentials.user = "kiqadmin";
  credentials.password = "kiqadmin";
  credentials.name = "kiq";

  DatabaseConfiguration configuration{};
  configuration.credentials = credentials;
  configuration.address = "127.0.0.1";
  configuration.port = "5432";

  return configuration;
}

void performQuery(DatabaseConnection* connection) {
  DatabaseQuery insert_query{};
  insert_query.type = QueryType::INSERT;
  insert_query.table = "apps";
  insert_query.fields = {"name", "path"};
  insert_query.values = {"kiggle", "/data/www/kiggle/index.js"};

  QueryResult insert_result = connection->query(insert_query);
}

FieldPairVector get_field_values(DatabaseConnection* connection,
                                 StringVec fields) {
  DatabaseQuery query{};
  query.type = QueryType::SELECT;
  query.table = "apps";
  query.fields = fields;
  FieldPairVector return_values{};
  QueryResult select_result = connection->query(query);
  for (const auto& value : select_result.values) {
    return_values.push_back({value.first, value.second});
  }

  return return_values;
}

KApplication retrieve_app(DatabaseConnection* connection) {
  FieldPairVector field_values =
      get_field_values(connection, {"path", "data", "name"});
  KApplication app{};
  for (const auto& value : field_values) {
    std::cout << value.first << ": " << value.second << std::endl;
    if (value.first == "path") {
      app.path = value.second;
    } else if (value.first == "data") {
      app.data = value.second;
    } else if (value.first == "name") {
      app.name = value.second;
    }
  }
  return app;
}

KApplication retrieve_app_by_name(DatabaseConnection& connection,
                                  const std::string& name) {
  DatabaseQuery select_query{.table = "apps",
                             .fields = {"name", "path", "data"},
                             .type = QueryType::SELECT,
                             .values = {},
                             .filter = {{"name", name}}};
  QueryResult result = connection.query(select_query);
  KApplication k_app{};
  for (const auto& row : result.values) {
    if (row.first == "name")
      k_app.name = row.second;
    else if (row.first == "path")
      k_app.path = row.second;

    else if (row.first == "data")
      k_app.data = row.second;
  }
  return k_app;
}

void write_file() {
  std::ofstream myfile{};
  myfile.open("example.txt");
  myfile << "Writing this to a file.\n";
  myfile.close();
}

std::string read_file(const char* path) {
  std::ifstream file{};
  std::string file_string{};
  file.open(path);
  if (file.is_open()) {
    while (getline(file, file_string)) {
      std::cout << file_string << std::endl;
    }
  }
  return file_string;
}

json load_file_as_json(const char* path) {
  std::ifstream file{path};
  return json::parse(file);
}

void save_app_to_database(DatabaseConnection& connection,
                          const std::string& name, const std::string& path,
                          const std::string& data) {
  DatabaseQuery insert_query{.table = "apps",
                             .fields = {"name", "path", "data"},
                             .type = QueryType::INSERT,
                             .values = {name, path, data}};
  connection.query(insert_query);
}

/**
 * start_process
 *
 * Starts a process and returns the stdout
 *
 * @param[in] {const char*} path The absolute path to the executable
 */
bool start_process(const char* path) {
  std::vector<std::string> v_args{};
  v_args.push_back(std::string(path));
  v_args.push_back(std::string("--test"));
  std::string returned_string = qx(v_args);
  std::cout << "Returned from process: " << returned_string << std::endl;
  if (returned_string.size() > 0) {
    return true;
  }
  return false;
}

int main(int argc, char** argv) {
  LOG(INFO) << "Logger initialized";
  DatabaseConnection connection;
  connection.setConfig(getConfiguration());

  KApplication collaborator_app =
      retrieve_app_by_name(connection, "collaborators");
  std::cout << collaborator_app << std::endl;
  /* KApplication app = retrieve_app(&connection); */

  /* std::future<bool> result_future = std::async(&start_process,
   * app.path.c_str()); */
  /* bool result = result_future.get(); */

  /* if (result) { */
  /*   // process data results */
  /*   json file_json = load_file_as_json(app.data.c_str()); */
  /*   KIQUtils::process_channel_info(file_json); */
  /* } */
  KiqServer::start_http_server();
  return 0;
}
